-----------------------------------------------------------------------------------------------
-- Client Lua Script for VisualLink
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
require "Vector3"
 
-----------------------------------------------------------------------------------------------
-- VisualLink Module Definition
-----------------------------------------------------------------------------------------------
local VisualLink = {} 

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------

local defaults = {
	Red = 1,
	Green = 1,
	Blue = 1,
	Alpha = 1
}

local isDocLoaded = false

defaults.__index = defaults

------
-----------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function VisualLink:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	self.markerSet = {}
	self.currentTarget = nil
	
	return o
end

function VisualLink:Init()
    Apollo.RegisterAddon(self)
end
 

-----------------------------------------------------------------------------------------------
-- VisualLink OnLoad
-----------------------------------------------------------------------------------------------

function VisualLink:OnRestore(eType, savedData)
	self.settings = self:CopyTable(savedData, self.settings)
end

function VisualLink:OnSave(eType)
    if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
	
	return self:CopyTable(self.settings)
end

function VisualLink:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("VisualLink.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)	

	Apollo.RegisterSlashCommand("vl", "OnSlash", self)	
	Apollo.RegisterSlashCommand("visuallink", "OnSlash", self)
	Apollo.RegisterEventHandler("MouseOverUnitChanged", "OnMouseOverUnitChanged", self)
	
	self.settings = self.settings or {}
	setmetatable(self.settings, defaults)
end

------------------------
-- VisualLink OnDocLoaded
-----------------------------------------------------------------------------------------------

function VisualLink:OnDocLoaded()	
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "VisualLinkForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)
	end
end


-----------------------------------------------------------------------------------------------
-- VisualLink Functions
-----------------------------------------------------------------------------------------------

function VisualLink:SetSliders()
	if isDocLoaded then
		
	end
end

function VisualLink:CopyTable(from, to)
	if not from then return end
        to = to or {}
	for k,v in pairs(from) do
		to[k] = v
	end
        return to
end

function VisualLink:GetColor()
	return CColor.new(self.settings.Red, self.settings.Green, self.settings.Blue, self.settings.Alpha)
end

function VisualLink:UpdateColorSlider(handler, control, newValue, oldValue)
	if handler ~= control then
        return
    end

	local value = newValue / 255
	local barname = handler:GetName()
	
	if barname == "redSliderBar" then	
		self.settings.Red = value
	elseif barname == "greenSliderBar" then	
		self.settings.Green = value
	elseif barname == "blueSliderBar" then	
		self.settings.Blue = value
	elseif barname == "alphaSliderBar" then	
		self.settings.Alpha = value
	end
		
	self.wndMain:FindChild("colorLabel"):SetTextColor(self:GetColor())	
end

function VisualLink:OnSlash()
	local color = self:GetColor()
	self.wndMain:FindChild("redSliderBar"):SetValue(color.r * 255)
	self.wndMain:FindChild("greenSliderBar"):SetValue(color.g * 255)
	self.wndMain:FindChild("blueSliderBar"):SetValue(color.b * 255)
	self.wndMain:FindChild("alphaSliderBar"):SetValue(color.a * 255)	
	self.wndMain:FindChild("colorLabel"):SetTextColor(color)
	self.wndMain:Show(true)
end

function VisualLink:OnMouseOverUnitChanged(target, num)
	self.currentTarget = target
	self:DrawLine()	
end

function VisualLink:DrawLine()
	for key, value in pairs(self.markerSet) do
		for i = 1, 10 do
			if self.markerSet[key][i] then 
				self.markerSet[key][i]:Destroy()
			end
		end
	end
	
	if self.currentTarget then	
		local clusterTargets = self.currentTarget:GetClusterUnits()
		local targetPosition = self.currentTarget:GetPosition()				

		if clusterTargets and targetPosition then
			local targetVector = Vector3.New(targetPosition.x, targetPosition.y, targetPosition.z)
				
			for key, value in ipairs(clusterTargets) do
				local clusterTargetPosition = value:GetPosition()
				
				local clusterTargetVector = Vector3.New(clusterTargetPosition.x, clusterTargetPosition.y, clusterTargetPosition.z)
				
				self.markerSet[key] = {}
								
				for i = 1, 10 do
					local fraction = i/10
					
					if self.markerSet[key][i] == nil then
						self.markerSet[key][i] = Apollo.LoadForm("VisualLink.xml", "Marker", nil, self)
						self.markerSet[key][i]:SetBGColor(self:GetColor())
					end
					
					self.markerSet[key][i]:SetWorldLocation(Vector3.InterpolateLinear(targetVector, clusterTargetVector, fraction))
					
					if self.markerSet[key][i]:IsOnScreen() then
						self.markerSet[key][i]:Show(true, true)
					else
						self.markerSet[key][i]:Show(false, true)
					end
				end
			end
		end
	end
end

-- when the Cancel button is clicked
function VisualLink:OnCancel()
	self.wndMain:Close() -- hide the window
	
end


-----------------------------------------------------------------------------------------------
-- VisualLink Instance
-----------------------------------------------------------------------------------------------
local VisualLinkInst = VisualLink:new()
VisualLinkInst:Init()
